package com.zanxiang.sdk.common.util;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

/**
 * @author : lingfeng
 * @time : 2021-11-23
 * @description : redis缓存通用前缀
 */
@Component
@Slf4j
public class RedisUtils<T> {

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 设置缓存
     *
     * @param key  : 缓存key
     * @param t    : 缓存对象
     * @param time : 过期时间
     * @return : 返回是否设置成功
     */
    public boolean setCache(String key, T t, long time) {
        try {
            ValueOperations<String, T> operations = redisTemplate.opsForValue();
            operations.set(key, t, time, TimeUnit.SECONDS);
            return true;
        } catch (Exception e) {
            log.error("缓存添加异常, key : {}, value : {}, e : {}", key, t, e.getMessage());
        }
        return false;
    }

    /**
     * 获取缓存方法
     *
     * @param key : 缓存key
     * @return : 返回缓存对象
     */
    public T getCache(String key) {
        try {
            if (redisTemplate.hasKey(key)) {
                ValueOperations<String, T> operations = redisTemplate.opsForValue();
                return operations.get(key);
            }
        } catch (Exception e) {
            log.error("获取缓存异常, key : {}, e : {}", key, e.getMessage());
        }
        return null;
    }

    /**
     * redis 事物锁设置方法
     *
     * @param key  : 缓存key
     * @param t    : 缓存对象
     * @param time : 过期时间
     * @return : 返回是否设置成功
     */
    public boolean setIfAbsent(String key, T t, long time) {
        try {
            ValueOperations<String, T> operations = redisTemplate.opsForValue();
            return operations.setIfAbsent(key, t, time, TimeUnit.SECONDS);
        } catch (Exception e) {
            log.error("缓存添加异常, key : {}, value : {}, e : {}", key, t, e.getMessage());
        }
        return false;
    }

    /**
     * 删除缓存的方法
     *
     * @param key : 缓存key
     * @return : 返回缓存对象
     */
    public boolean deleteCache(String key) {
        try {
            if (redisTemplate.hasKey(key)) {
                return redisTemplate.delete(key);
            }
        } catch (Exception e) {
            log.error("删除缓存异常, key : {}, e : {}", key, e.getMessage());
        }
        return false;
    }
}
