package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.mybatis.entity.Order;
import com.zanxiang.mybatis.mapper.OrderMapper;
import com.zanxiang.sdk.domain.bo.PlatformOrderBO;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.service.PlatformOrderService;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.extension.ParameterResolutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * 平台订单逻辑开发
 *
 * @author xufeng
 * @date 2022-06-06 17:06
 */

@Component
@Slf4j
@Service
public class PlatformOrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements PlatformOrderService {
    protected final Logger logger = LoggerFactory.getLogger(PlatformOrderServiceImpl.class);

    @Autowired
    private OrderMapper orderMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public String create(PlatformOrderBO platformOrderBO) {
        try {
            logger.info("生成订单请求参数 platformOrderBO:{}", platformOrderBO);
            this.checkParam(platformOrderBO, "create");
            Order data = BeanUtils.copy(platformOrderBO, Order.class);
            if (save(data)) {
                return data.getId();
            }
        } catch (Exception e) {
            logger.error("生成订单失败 platformOrderBO:{}, error:{}", platformOrderBO, e);
            e.printStackTrace();
        }
        return null;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean pay(PlatformOrderBO platformOrderBO) {
        logger.info("订单支付请求参数 platformOrderBO:{}", platformOrderBO);
        try {
            this.checkParam(platformOrderBO, "pay");
            Order order = getOne(new LambdaQueryWrapper<Order>().eq(Order::getId, platformOrderBO.getId()));
            //已支付情况，直接返回成功
            if (!order.getStatus().equals(OrderStateEnum.SUCCESS.getCode())) {
                return true;
            }
            if (!order.getStatus().equals(OrderStateEnum.NO_PAY.getCode())) {
                throw new ParameterResolutionException("订单状态非待支付");
            }
            if (!order.getUserId().equals(platformOrderBO.getUserId())) {
                throw new ParameterResolutionException("订单与用户关系不符");
            }
            order.setRealAmount(platformOrderBO.getRealAmount());
            order.setMerchantOrderNo(platformOrderBO.getMerchantOrderNo());
            order.setStatus(OrderStateEnum.SUCCESS.getCode());
            order.setGamePaywayId(platformOrderBO.getGamePaywayId());
            order.setPayTime(platformOrderBO.getPayTime() == null ? platformOrderBO.getPayTime() : new Date());
            order.setMemNote(StringUtils.isEmpty(platformOrderBO.getMemNote()) ? null : platformOrderBO.getMemNote());
            logger.info("订单支付提交数据 data:{}", order);
            return updateById(order);
        } catch (ParameterResolutionException e) {
            logger.error("订单支付失败 platformOrderBO:{}, error:{}", platformOrderBO, e);
            e.printStackTrace();
            return false;
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean cancel(PlatformOrderBO platformOrderBO) {
        logger.info("订单支付取消请求参数 platformOrderBO:{}", platformOrderBO);
        try {
            this.checkParam(platformOrderBO, "cancel");
            Order order = new Order();
            order.setId(platformOrderBO.getId());
            order.setStatus(OrderStateEnum.CANCEL.getCode());
            order.setAdminNote(StringUtils.isEmpty(platformOrderBO.getAdminNote()) ? null : platformOrderBO.getAdminNote());
            order.setRemark(StringUtils.isEmpty(platformOrderBO.getRemark()) ? null : platformOrderBO.getRemark());
            logger.info("订单取消提交数据 data:{}", order);
            return updateById(order);
        } catch (Exception e) {
            logger.error("订单取消失败 platformOrderBO:{}, error:{}", platformOrderBO, e);
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public PlatformOrderDTO info(String id, String userId) {
        Order order = orderMapper.selectByPrimaryKey(Long.valueOf(id));
//        Order order = getOne(new LambdaQueryWrapper<Order>().eq(Order::getId, id).eq(Order::getUserId, userId));
        return BeanUtils.copy(order, PlatformOrderDTO.class);
    }

    /**
     * 检查订单参数
     *
     * @param platformOrderBO
     * @return
     */
    private Boolean checkParam(PlatformOrderBO platformOrderBO, String active) {
        switch (active) {
            case "create":
                if (platformOrderBO.getCpId() == null) {
                    throw new NullPointerException("CpId");
                }
                if (platformOrderBO.getUserId() == null) {
                    throw new NullPointerException("UserId");
                }
                if (platformOrderBO.getMgUserId() == null) {
                    throw new NullPointerException("MgUserId");
                }
                if (StringUtils.isEmpty(platformOrderBO.getProductId())) {
                    throw new NullPointerException("ProductId");
                }
                if (StringUtils.isEmpty(platformOrderBO.getProductName())) {
                    throw new NullPointerException("ProductName");
                }
                if (platformOrderBO.getProductCnt() == null) {
                    throw new NullPointerException("ProductCnt");
                }
                if (platformOrderBO.getAmount() == null) {
                    throw new NullPointerException("Amount");
                }
                break;
            case "pay":
                if (platformOrderBO.getId() == null) {
                    throw new NullPointerException("id");
                }
                if (platformOrderBO.getUserId() == null) {
                    throw new NullPointerException("UserId");
                }
                if (platformOrderBO.getStatus() == 0) {
                    throw new NullPointerException("Status");
                }
                if (StringUtils.isEmpty(platformOrderBO.getGamePaywayId())) {
                    throw new NullPointerException("GamePaywayId");
                }
                if (platformOrderBO.getRealAmount() == null) {
                    throw new NullPointerException("RealAmount");
                }
                if (StringUtils.isEmpty(platformOrderBO.getMerchantOrderNo())) {
                    throw new NullPointerException("MerchantOrderNo");
                }


                break;

            case "cancel":
                if (platformOrderBO.getId() == null) {
                    throw new NullPointerException("id");
                }

                break;
            default:
                return false;
        }
        return true;
    }


}
